//!
//!
use super::{models, schema::member_permissions};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_permissions").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_resource: &models::NewMemberPermission,
  ) -> Result<models::MemberPermission, Error> {
    let inserted = diesel::insert_into(member_permissions::table)
      .values(new_resource)
      .get_result::<models::MemberPermission>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn inserts(
    &self,
    conn: &diesel::PgConnection,
    new_resource_actions: &Vec<models::NewMemberPermission>,
  ) -> Result<Vec<models::MemberPermission>, Error> {
    let inserted = diesel::insert_into(member_permissions::table)
      .values(new_resource_actions)
      .get_results(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
    role_resource_action_id: uuid::Uuid,
  ) -> Result<Option<models::MemberPermission>, Error> {
    use member_permissions::dsl;

    match member_permissions::table
      .filter(
        dsl::member_id
          .eq(member_id)
          .and(dsl::role_resource_action_id.eq(role_resource_action_id)),
      )
      .first::<models::MemberPermission>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_permissions::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_permissions::dsl::member_id.eq(sp));
      }
      if let Some(sp) = s.role_resource_action_id {
        q = q.filter(member_permissions::dsl::role_resource_action_id.eq(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::MemberPermission>, Error> {
    let mut q = member_permissions::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_permissions::dsl::member_id.eq(sp));
      }
      if let Some(sp) = s.role_resource_action_id {
        q = q.filter(member_permissions::dsl::role_resource_action_id.eq(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "role_resource_action_id" => {
              q = q.order_by(member_permissions::role_resource_action_id.asc());
            }
            "member_id" => {
              q = q.order_by(member_permissions::member_id.asc());
            }
            "created_at" => {
              q = q.order_by(member_permissions::created_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "role_resource_action_id" => {
              q = q.order_by(member_permissions::role_resource_action_id.desc());
            }
            "member_id" => {
              q = q.order_by(member_permissions::member_id.desc());
            }
            "created_at" => {
              q = q.order_by(member_permissions::created_at.desc());
            }

            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberPermission>(conn)
  }

  ///
  pub fn delete(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
    role_resource_action_id: uuid::Uuid,
  ) -> Result<u64, Error> {
    use member_permissions::dsl;

    diesel::delete(
      member_permissions::table.filter(
        dsl::member_id
          .eq(member_id)
          .and(dsl::role_resource_action_id.eq(role_resource_action_id)),
      ),
    )
    .execute(conn)
    .map(|c| c as u64)
  }

  ///
  pub fn delete_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<u64, Error> {
    use member_permissions::dsl;

    diesel::delete(member_permissions::table.filter(dsl::member_id.eq(member_id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
